(* ------------------------------------------------------------------------ *)
(*                                                                          *)
(*   Pulldown-Menues  PDM 1.0                                               *)
(*       Turbo Pascal Version 2.0                                           *)
(*       Oliver Benke                                     05.05.1988        *)
(*                                                                          *)
(* ------------------------------------------------------------------------ *)

(*$V-   Stringlaengenpruefung lockern *)

const MaxItems          = 10;  (* Max. Menuepunkte *)
      ItemLen           = 25;  (* Max. Laenge jedes Menuepunktes *)
      OverheadLen       = 25;  (* Max. Laenge des Titels *)

      SaveScreen  = $0046;  (* RSX-Funktionen *)
      LoadScreen  = $0146;
      SAdr        = $0246;
      norm        = 0;
      inv         = $2f;  (* Z80-Befehl CPL/ selbstmodifizierend... *)

type PDMType = record
                 xpos,ypos        : byte;
                 overhead         : string[OverheadLen];
                 NoItems          : byte;  (* Anzahl tatsaechlich vorhandener
                                              Menuepunkte *)
                 Item             : array[1..MaxItems] of string[ItemLen];
                 LastItem         : integer
                                  (* Zuletzt gewaehlter Menuepunkt *)
               end;

     ParType = record  (* Eine derartige Struktur wird von der RSX-Funktion
                          >SaveScreen< erwartet. *)
                 zeile, spalte, hoehe, breite: byte
               end;

     str90 = string[90];
     ConOutType = array[1..58] of byte;

var ConOut: ConOutType;
    par: ParType;
    fkt: integer;  (* RSX-Funktion *)

(* ======================================================================== *)
(* Maschinennahe Prozeduren *)

procedure ConOutInit;
(* Kopiert den MCode in die Variable ConOut, also an eine der oberen Adressen
   im Speicher. *)

const MCode: ConOutType =
      ($F3,$3E,$81,$D3,$F1,$3C,$D3,$F2,$7A,$16,$00,$07,$17,$17,$17,$6F,
       $7A,$CE,$B6,$67,$7E,$23,$66,$6F,$EB,$29,$29,$19,$29,$E5,$69,$26,
       $00,$29,$29,$29,$11,$00,$B8,$19,$D1,$06,$08,$7E,$00,$12,$23,$13,
       $10,$F9,$3E,$85,$D3,$F1,$3C,$D3,$F2,$C9);

begin
  if addr(ConOut) < $c000 then begin
    ClrScr; WriteLn ('Speicherueberlauf in PDM.'); halt
  end;
  ConOut := MCode
end;

(* ------------------------------------------------------------------------ *)

procedure WriteXY (x,y: byte; str: str90);
(* Schnelle Bildschirmausgabe *)

var xt,yt,i: integer;
    c      : char;

begin
  xt := pred(x); yt := pred(y);  (* Koordinatensystem wie Turbo Pascal... *)
  for i:=1 to length(str) do begin
    c := str[i];
    inline ($3a/yt/$57/$3a/xt/$5f/$3a/c/$4f/$cd/ConOut);
    xt := succ(xt)
  end
end;

(* ------------------------------------------------------------------------ *)

procedure rsx;
(* RSX-Funktion ausfuehren *)

begin
  inline ($01/par/$21/fkt/$11/fkt/$cd/5/0)
end;

(* ------------------------------------------------------------------------ *)

const up    = 14;  (* Tastaturnummer Cursor hoch          *)
      down  = 79;  (*                Cursor runter        *)
      left  = 15;  (*                Cursor links         *)
      right = 06;  (*                Cursor rechts        *)
      cr    = 18;  (*                RETURN               *)
      enter = 78;  (*                ENTER                *)
      can   = 75;  (*                CAN/ letztes Element *)

var b: byte absolute $fba3;  (* Tastaturstatus *)
    c: byte absolute $fba2;  (* Tastennummer *)

procedure GetKey;
(* Tastaturpuffer ueber XBIOS lesen *)

begin
  inline ($cd/$fc5a/$da/$00/$d2/*-6/$ed/$43/c)
end;

(* ======================================================================== *)

function PullDown (var pdm: PDMType): integer;
(* Gibt Pulldown-Menue aus und verwertet die Eingaben
   Rueckgabeparameter: gewaehlter Menuepunkt  *)

var i,p,d: integer;

   procedure MoveWindow; FORWARD;

   procedure DrawFrame (x,y,h,b: integer);
   (* Rahmen zeichnen und Ueberschrift ausgeben *)

   begin
     WriteXY (x,y,#134); WriteXY (x,y+pred(h),#131);
     for i:=1 to pred(b) do begin
       WriteXY (x+i,y,#138); WriteXY (x+b-i,y+pred(h),#138);
       WriteXY (x+i,y+2,#138)
     end;
     WriteXY (x+i,y,#140); WriteXY (x+i,y+pred(h),#137);
     WriteXY (x,y+2,#135); WriteXY (x+i,y+2,#141);
     for i:=1 to h-2 do begin
       WriteXY (x,y+i,#133);
       WriteXY (x+pred(b),y+pred(h)-i,#133)
     end;
     WriteXY (succ(x),succ(y),pdm.overhead);
   end;

   procedure MoveBar;
   (* Balken bewegen *)

   begin
     if d <> 0 then begin
       ConOut[45] := inv; WriteXY (succ(pdm.xpos),pdm.ypos+2+p,pdm.item[p])
     end;
     if p+d > pdm.NoItems then p := (p+d) mod pdm.NoItems
     else if p+d < 1 then p := pdm.NoItems + p+d else p := p+d;
     ConOut[45] := norm; WriteXY (succ(pdm.xpos),pdm.ypos+2+p,pdm.item[p])
   end;

   procedure DisplayMenu;
   (* Bildschirm sichern und Menue aufbauen *)

   begin
     fkt := SaveScreen; rsx;
     ConOut[45] := inv;
     for i:=1 to pdm.NoItems do
       WriteXY (succ(pdm.xpos),pdm.ypos+2+i,pdm.item[i]);
     DrawFrame (pdm.xpos,pdm.ypos,par.hoehe,par.breite);
     p := pdm.LastItem; d := 0; MoveBar;
   end;

   procedure choice;
   (* Tastatureingabe auswerten *)

   begin
     repeat
       GetKey;
       case c of
         up  : d := -1;
         down: d := 1;
         left: d := -2;
         right:d := 2;
         can : begin
                 p := pdm.NoItems; c := cr; d := 0
               end
         else d := 0;
       end;  (* case *)
       if d <> 0 then
         if B > 127 then MoveWindow else MoveBar  (* ALT gedrueckt? *)
     until (c=cr) or (c=enter)
   end;

   procedure MoveWindow;  (* FORWARD deklariert ... *)
   (* Window verschieben *)

   begin
     if abs(d)=1 then par.zeile := par.zeile + (d shl 2)
     else par.spalte := par.spalte + (d shl 3);
     if (par.zeile < 2) or (par.zeile+par.hoehe > 30) then par.zeile := 3;
     if (par.spalte < 2) or (par.spalte+par.breite > 90) then par.spalte := 3;
     pdm.xpos := succ(par.spalte); pdm.ypos := succ(par.zeile);
     fkt := LoadScreen; rsx;
     pdm.LastItem := p;
     DisplayMenu
   end;

begin  (* PullDown *)
  ConOutInit;
  par.spalte := pred(pdm.xpos);            par.zeile := pred(pdm.ypos);
  par.breite := length (pdm.item[1]) + 2;  par.hoehe := pdm.NoItems + 4;
  while length(pdm.overhead) < length(pdm.item[1]) do pdm.overhead := pdm.overhead + ' ';
  DisplayMenu; choice;
  fkt := LoadScreen; rsx;
  pdm.LastItem := p; PullDown := p  (* Parameter rueckgabe *)
end;

(* ------------------------------------------------------------------------ *)
(*                        Ende von PDM.PAS                                  *)
