;-----------------------------------------------------------------------------
;
;  WINDOWS.ASM  8080 RSX Version 1.3
;  Bildschirmfenster sichern bzw. wiederherstellen
;      Oliver Benke                                         05.05.1988
;
;
;  Parameter:      DE & HL       Adresse der Variable, welche die Funktions-
;                                nummer enthaelt (WINDOWS = 70)
;                  BC            Adresse vom Parameterblock
;                  Diese BASIC-freundliche Parameteruebergabe entspricht der
;                  von R. Herrmann in PC Int. 12'87 verwendeten.
;
;  ParameterBlock: 1. Byte       Zeile
;                  2. Byte       Spalte
;                  3. Byte       Hoehe
;                  4. Byte       Breite
;
;  - alle Standartregister werden zerstoert -
;
;-----------------------------------------------------------------------------

ldir          equ 0b0edh          ; Z80-Befehl

;-----------------------------------------------------------------------------
; RSX Prefix

              ds   6
              jmp  ftest
next          jmp  $-$
              dw   0
              dw   0ffh
              db   'PDM     '
              ds   3

;-----------------------------------------------------------------------------
; RSX Filter

ftest         mov  a,m
              cpi  70             ; Funktion <> 70 ?
              jnz  next           ;  -> Normale BDOS-Routine
              ldax d              ; Zweiter Test...
              cpi  70
              jnz  next
begin         dw   073edh         ; ld (stack),sp
              dw   stack
              lxi  sp,StackEnd
              call RSXFkt
              dw   07bedh         ; ld sp,(stack)
              dw   stack
              ret
RSXFkt        inx  h
              mov  a,m            ; 0 -> SaveScreen, 1 -> LoadScreen
              ora  a
              jnz  LoadScreen

;-----------------------------------------------------------------------------
; Bildschirmbereich sichern

SaveScreen    push b
              lda  PDMactive      ; PDMactive = TRUE ?
              ora  a              ; cpi 0
              cz   LoadScreen     ;  -> altes Menue zunaechst schliessen
              xra  a              ; mvi a,0
              sta  PDMactive      ; PDMactive := TRUE
              pop  h              ; Adresse des Parameterblocks (von BC)
              lxi  d,zeile        ; Parameterblock in lokale Variable kopieren
              lxi  b,4            ; (call by value)
              push d
              push b
              dw   ldir
              pop  b              ; BC -> 4
              pop  h              ; HL -> zeile -> Arbeits-Parameterblock
              lxi  d,par2         ; DE -> par2 -> >LoadScreen<
              dw   ldir           ; Parameterblock fuer >LoadScreen< kopieren
              mvi  a,81h          
              call memory         ; Screenenvironment einblenden
              lda  hoehe
              mov  b,a
loop          push b  
              call SAdr
              mov  a,h
              sui  40h            ; Adresse verschieben
              mov  h,a
              dw   05bedh         ; ld de,(PDMAdr)
              dw   PDMAdr
              call move
              dw   053edh         ; ld de,(PDMAdr)
              dw   PDMAdr
              lxi  h,zeile        ; (zeile) := (zeile) + 1
              inr  m
              pop  b
              dcr  b              ; djnz loop
              jnz  loop
              lxi  h,buffer       ; >PDMAdr2< fuer >LoadScreen< aktualisieren
              shld PDMAdr2
              jmp  ende

;-----------------------------------------------------------------------------
; Bildschirmbereich wiederherstellen

LoadScreen    lxi  h,par2         ; Parameterblock >par2<
              lxi  d,zeile        ; nach Arbeits-Parameterblock
              lxi  b,4            ; kopieren...
              dw   ldir
              mvi  a,0ffh         ; PDMactive := FALSE
              sta  PDMactive
              mvi  a,81h          ; Screenenvironment einblenden
              call memory
              lda  hoehe
              mov  b,a
loop2         push b
              call SAdr
              mov  a,h
              sui  40h            ; Adresse verschieben
              mov  h,a
              xchg
              lhld PDMAdr2
              call move           ; Von >PDMAdr2< nach >SAdr< kopieren
              shld PDMAdr2
              lxi  h,zeile        ; (zeile) := (zeile) + 1
              inr  m
              pop  b
              dcr  b
              jnz  loop2
              lxi  h,buffer       ; >PDMAdr< fuer >SaveScreen< kopieren
              shld PDMAdr

;-----------------------------------------------------------------------------
; Ende von SaveScreen/ LoadScreen

ende          mvi  a,84h          ; 1. Block der TPA
memory        di                  ; Interrupts sperren
              out  0f0h           ; Z80-Adressbereich 0000-3FFF
              inr  a
              out  0f1h           ;                   4000-7FFF 
              ret

;-----------------------------------------------------------------------------
; Unterroutine von SaveScreen/ LoadScreen

move          lda  breite         ; Anzahl Zeichen (1 Zeichen => 8 Bytes)
              ral                 ; A := A * 2
              ral                 ; A := A * 2 * 2
              ral                 ; A := A * 2 * 2 * 2  =>  A * 8
              mov  c,a            ; Anzahl der zu kopierenden Bytes berechnen
              mvi  b,0
              dw   ldir
              ret

;-----------------------------------------------------------------------------
; Adresse im Bildschirmspeicher berechnen
; Nach M. Uphoff, PC Int. 6'87

SAdr          lda  spalte
              mov  e,a
              mvi  d,0
              lda  zeile
              rlc
              ral
              ral
              ral
              mov  l,a
              mov  a,d
              aci  076h
              mov  h,a
              mov  a,m
              inx  h
              mov  h,m
              mov  l,a
              xchg
              dad  h
              dad  h
              dad  d
              dad  h
              ret

;-----------------------------------------------------------------------------
; Variablen:

zeile         ds   1              ; Parameterblock 1
spalte        ds   1              ;        "
hoehe         ds   1              ;        "
breite        ds   1              ;        "
par2          ds   4              ; Parameterblock 2
PDMAdr        dw   buffer         ; Puffer-Adresse >SaveScreen<
PDMAdr2       dw   buffer         ;                >LoadScreen<
PDMactive     db   0ffh           ; Flag

;====================================================================
; In >buffer< wird festgelegt, wie gross der Puffer fuer die Windows
; maximal sein darf. Brauchen Sie z.B. ein 1024 Zeichen grosses 
; Window, so muss hier >ds 2000h< stehen... (Verfuegbare Zeichen = 
; Anzahl Bytes durch 8; 2000 hexadezimal ist das selbe wie 8192
; dezimal) 

buffer        ds   1000h          ; beliebig erweiterbar...

;====================================================================

stack         ds   36             ; Stack fuer die RSX-Routine
StackEnd

end
